/* cdplayCmds.h : generated by build_cmd, do not edit */

extern char *hp_cd_continue();
extern char *hp_cd_eject();
extern char *hp_cd_init();
extern char *hp_cd_pause();
extern char *hp_cd_play();
extern char *hp_cd_playstatus();
extern char *hp_cd_search();
extern char *hp_cd_trackinfo();
extern int playerCmd(/* */);
extern int playerRetStrCmd(/* */);
extern int playerTrkCmd(/* */);
