/* init_cdplayCmds.c : generated by build_cmd, do not edit */

#include <stdio.h>
#include <tkInt.h>
#include <tcl.h>
#include "cdplayCmds.h"

int
init_cdplayCmds(interp)
Tcl_Interp *interp;
{
	Tcl_CreateCommand(interp, "cd_init", playerCmd,
		(ClientData) hp_cd_init, (void (*)()) NULL);
	Tcl_CreateCommand(interp, "cd_play", playerTrkCmd,
		(ClientData) hp_cd_play, (void (*)()) NULL);
	Tcl_CreateCommand(interp, "cd_playstatus", playerRetStrCmd,
		(ClientData) hp_cd_playstatus, (void (*)()) NULL);
	Tcl_CreateCommand(interp, "cd_trackinfo", playerRetStrCmd,
		(ClientData) hp_cd_trackinfo, (void (*)()) NULL);
	Tcl_CreateCommand(interp, "cd_pause", playerCmd,
		(ClientData) hp_cd_pause, (void (*)()) NULL);
	Tcl_CreateCommand(interp, "cd_continue", playerCmd,
		(ClientData) hp_cd_continue, (void (*)()) NULL);
	Tcl_CreateCommand(interp, "cd_search", playerTrkCmd,
		(ClientData) hp_cd_search, (void (*)()) NULL);
	Tcl_CreateCommand(interp, "cd_eject", playerCmd,
		(ClientData) hp_cd_eject, (void (*)()) NULL);

	return 0;
}
